@@ -1,4 +1,4 @@
-"""Hot start helper functions."""
+"""Warm start and hot start helper functions."""
 
 import numpy as np
 import scipy.stats
@@ -47,7 +47,7 @@ def get_auxiliary_problem(loglike, transform, ctr, invcov, enlargement_factor, d
         The default is recommended. For truly gaussian posteriors,
         the student-t can be made more gaussian (by df>=30) for accelation.
 
-    Returns:
+    Returns
     ---------
     aux_loglike: function
         auxiliary loglikelihood function.
@@ -78,8 +78,8 @@ def get_auxiliary_problem(loglike, transform, ctr, invcov, enlargement_factor, d
         if not (x > 0).all() or not (x < 1).all():
             return -1e300
         # undo the effect of the auxiliary distribution
-        loglike = rv_auxiliary1d.logpdf(coords).sum()
-        return loglike(transform(x)) - loglike
+        loglike_total = rv_auxiliary1d.logpdf(coords).sum()
+        return loglike(transform(x)) - loglike_total
 
     def aux_aftertransform(u):
         return transform(aux_rotator(rv_auxiliary1d.ppf(u)))
@@ -122,7 +122,7 @@ def get_extended_auxiliary_problem(loglike, transform, ctr, invcov, enlargement_
         The default is recommended. For truly gaussian posteriors,
         the student-t can be made more gaussian (by df>=30) for accelation.
 
-    Returns:
+    Returns
     ---------
     aux_loglike: function
         auxiliary loglikelihood function. Takes d + 1 parameters (see below).
@@ -204,7 +204,7 @@ def get_extended_auxiliary_independent_problem(loglike, transform, ctr, err, df=
         The default is recommended. For truly gaussian posteriors,
         the student-t can be made more gaussian (by df>=30) for accelation.
 
-    Returns:
+    Returns
     ---------
     aux_loglike: function
         auxiliary loglikelihood function.
@@ -242,6 +242,213 @@ def get_extended_auxiliary_independent_problem(loglike, transform, ctr, err, df=
     return aux_loglikelihood, aux_transform
 
 
+def compute_quantile_intervals(steps, upoints, uweights):
+    """Compute lower and upper axis quantiles.
+
+    Parameters
+    ------------
+    steps: array
+        list of quantiles q to compute.
+    upoints: array
+        samples, with dimensions (N, d)
+    uweights: array
+        sample weights
+
+    Returns
+    ---------
+    ulo: array
+        list of lower quantiles (at q), one entry for each dimension d.
+    uhi: array
+        list of upper quantiles (at 1-q), one entry for each dimension d.
+    """
+    ndim = upoints.shape[1]
+    nboxes = len(steps)
+    ulos = np.empty((nboxes + 1, ndim))
+    uhis = np.empty((nboxes + 1, ndim))
+    for j, pthresh in enumerate(steps):
+        for i, ui in enumerate(upoints.transpose()):
+            order = np.argsort(ui)
+            c = np.cumsum(uweights[order])
+            usel = ui[order][np.logical_and(c >= pthresh, c <= 1 - pthresh)]
+            ulos[j,i] = usel.min()
+            uhis[j,i] = usel.max()
+    ulos[-1] = 0
+    uhis[-1] = 1
+    return ulos, uhis
+
+
+def compute_quantile_intervals_refined(steps, upoints, uweights, logsteps_max=20):
+    """Compute lower and upper axis quantiles.
+
+    Parameters
+    ------------
+    steps: array
+        list of quantiles q to compute, with dimensions
+    upoints: array
+        samples, with dimensions (N, d)
+    uweights: array
+        sample weights. N entries.
+    logsteps_max: int
+        number of intermediate steps to inject between largest quantiles interval and full unit cube
+
+    Returns
+    ---------
+    ulo: array
+        list of lower quantiles (at `q`), of shape (M, d), one entry per quantile and dimension d.
+    uhi: array
+        list of upper quantiles (at 1-`q`), of shape (M, d), one entry per quantile and dimension d.
+    uinterpspace: array
+        list of steps (length of `steps` plus `logsteps_max` long)
+    """
+    nboxes = len(steps)
+    ulos_orig, uhis_orig = compute_quantile_intervals(steps, upoints, uweights)
+    assert len(ulos_orig) == nboxes + 1
+    assert len(uhis_orig) == nboxes + 1
+
+    smallest_axis_width = np.min(uhis_orig[-2,:] - ulos_orig[-2,:])
+    logsteps = min(logsteps_max, int(np.ceil(-np.log10(max(1e-100, smallest_axis_width)))))
+
+    weights = np.logspace(-logsteps, 0, logsteps + 1).reshape((-1, 1))
+    # print("logspace:", weights, logsteps)
+    assert len(weights) == logsteps + 1, (weights.shape, logsteps)
+    # print("quantiles:", ulos_orig, uhis_orig)
+    ulos_new = ulos_orig[nboxes - 1, :].reshape((1, -1)) * (1 - weights) + 0 * weights
+    uhis_new = uhis_orig[nboxes - 1, :].reshape((1, -1)) * (1 - weights) + 1 * weights
+
+    # print("additional quantiles:", ulos_new, uhis_new)
+
+    ulos = np.vstack((ulos_orig[:-1,:], ulos_new))
+    uhis = np.vstack((uhis_orig[:-1,:], uhis_new))
+    # print("combined quantiles:", ulos, uhis)
+    assert (ulos[-1,:] == 0).all()
+    assert (uhis[-1,:] == 1).all()
+
+    uinterpspace = np.ones(nboxes + logsteps + 1)
+    uinterpspace[:nboxes + 1] = np.linspace(0, 1, nboxes + 1)
+    assert 0 < uinterpspace[nboxes - 1] < 1, uinterpspace[nboxes]
+    uinterpspace[nboxes:] = np.linspace(uinterpspace[nboxes - 1], 1, logsteps + 2)[1:]
+
+    return ulos, uhis, uinterpspace
+
+
+def get_auxiliary_contbox_parameterization(
+    param_names, loglike, transform, upoints, uweights, vectorized=False,
+):
+    """Return a new loglike and transform based on an auxiliary distribution.
+
+    Given a likelihood and prior transform, and information about
+    the (expected) posterior peak, generates a auxiliary
+    likelihood and prior transform that is identical but
+    requires fewer nested sampling iterations.
+
+    This is achieved by deforming the prior space, and undoing that
+    transformation by correction weights in the likelihood.
+    A additional parameter, "aux_logweight", is added at the end,
+    which contains the correction weight. You can ignore it.
+
+    The auxiliary distribution used for transformation/weighting is
+    factorized. Each axis considers the ECDF of the auxiliary samples,
+    and segments it into quantile segments. Within each segment,
+    the parameter edges in u-space are linearly interpolated.
+    To see the interpolation quantiles for each axis, use::
+
+        steps = 10**-(1.0 * np.arange(1, 8, 2))
+        ulos, uhis, uinterpspace = compute_quantile_intervals_refined(steps, upoints, uweights)
+
+    Parameters
+    ------------
+    param_names: list
+        parameter names
+    loglike: function
+        original likelihood function
+    transform: function
+        original prior transform function
+    upoints: array
+        Posterior samples (in u-space).
+    uweights: array
+        Weights of samples (needs to sum of 1)
+    vectorized: bool
+        whether the loglike & transform functions are vectorized
+
+    Returns
+    ---------
+    aux_param_names: list
+        new parameter names (`param_names`) plus additional 'aux_logweight'
+    aux_loglike: function
+        auxiliary loglikelihood function.
+    aux_transform: function
+        auxiliary transform function.
+        Takes d u-space coordinates, and returns d + 1 p-space parameters.
+        The first d return coordinates are identical to what ``transform`` would return.
+        The final coordinate is the log of the correction weight.
+      vectorized: bool
+        whether the returned functions are vectorized
+
+    Usage
+    ------
+    ::
+
+        aux_loglikelihood, aux_transform = get_auxiliary_contbox_parameterization(
+            loglike, transform, auxiliary_usamples)
+        aux_sampler = ReactiveNestedSampler(parameters, aux_loglikelihood, transform=aux_transform, derived_param_names=['logweight'])
+        aux_results = aux_sampler.run()
+        posterior_samples = aux_results['samples'][:,-1]
+
+    """
+    mask = np.logical_and(upoints > 0, upoints < 1)
+    assert np.all(mask), (
+        'upoints must be between 0 and 1, have:', upoints[~mask,:])
+    steps = 10**-(1.0 * np.arange(1, 8, 2))
+    nsamples, ndim = upoints.shape
+    assert nsamples > 10
+    ulos, uhis, uinterpspace = compute_quantile_intervals_refined(steps, upoints, uweights)
+
+    aux_param_names = param_names + ['aux_logweight']
+
+    def aux_transform(u):
+        ndim2, = u.shape
+        assert ndim2 == ndim + 1
+        umod = np.empty(ndim)
+        log_aux_volume_factors = 0
+        for i in range(ndim):
+            ulo_here = np.interp(u[-1], uinterpspace, ulos[:,i])
+            uhi_here = np.interp(u[-1], uinterpspace, uhis[:,i])
+            umod[i] = ulo_here + (uhi_here - ulo_here) * u[i]
+            log_aux_volume_factors += np.log(uhi_here - ulo_here)
+        return np.append(transform(umod), log_aux_volume_factors)
+
+    def aux_transform_vectorized(u):
+        nsamples, ndim2 = u.shape
+        assert ndim2 == ndim + 1
+        umod = np.empty((nsamples, ndim2 - 1))
+        log_aux_volume_factors = np.zeros((nsamples, 1))
+        for i in range(ndim):
+            ulo_here = np.interp(u[:,-1], uinterpspace, ulos[:,i])
+            uhi_here = np.interp(u[:,-1], uinterpspace, uhis[:,i])
+            umod[:,i] = ulo_here + (uhi_here - ulo_here) * u[:,i]
+            log_aux_volume_factors[:,0] += np.log(uhi_here - ulo_here)
+        return np.hstack((transform(umod), log_aux_volume_factors))
+
+    def aux_loglikelihood(x):
+        x_actual = x[:-1]
+        logl = loglike(x_actual)
+        aux_logweight = x[-1]
+        # downweight if we are in the auxiliary distribution
+        return logl + aux_logweight
+
+    def aux_loglikelihood_vectorized(x):
+        x_actual = x[:,:-1]
+        logl = loglike(x_actual)
+        aux_logweight = x[:,-1]
+        # downweight if we are in the auxiliary distribution
+        return logl + aux_logweight
+
+    if vectorized:
+        return aux_param_names, aux_loglikelihood_vectorized, aux_transform_vectorized, vectorized
+    else:
+        return aux_param_names, aux_loglikelihood, aux_transform, vectorized
+
+
 def reuse_samples(
     param_names, loglike, points, logl, logw=None,
     logz=0.0, logzerr=0.0, upoints=None,
@@ -276,7 +483,7 @@ def reuse_samples(
     log_weight_threshold: float
         Lowest log-weight to consider
 
-    Returns:
+    Returns
     ---------
     results: dict
         All information of the run. Important keys:
